# coding: utf-8
u"""Валидаторы для простых (текстовых, числовых и т.п.) полей модели Django."""
from functools import partial
from itertools import izip
import re

from django.core.exceptions import ValidationError

from . import validate_value


# =============================================================================
# СНИЛС
# =============================================================================


_snils_re = re.compile(r'^\d{3}-\d{3}-\d{3} \d{2}$')  # СНИЛС


def snils_validator(value):
    u"""Валидатор СНИЛС.

    Проверяет корректность формата и контрольное число (последние две цифры).

    :raises django.core.exceptions.ValidationError: Если аргумент value
        содержит значение, несоответствующее формату СНИЛС, либо контрольное
        число некорректно.
    """
    value = unicode(value)

    if not _snils_re.match(value):
        raise ValidationError(u'СНИЛС должен быть в формате ###-###-### ##')

    if value[:11] <= u'001-001-998':
        return

    numbers = (int(ch) for ch in reversed(value[:11]) if ch.isdigit())
    checksum = int(value[12:14])
    summa = sum(i * n for i, n in enumerate(numbers, 1))
    if summa > 101:
        summa %= 101

    if (summa < 100 and summa != checksum or
            summa in (100, 101) and checksum != 0):
        raise ValidationError(u'не пройдена проверка контрольного числа')


is_snils_valid = partial(validate_value, validator=snils_validator)


# =============================================================================
# ИНН
# =============================================================================


def _check_inn_checksum(coefficients, numbers, checksum):
    summa = sum(c * n for c, n in izip(coefficients, numbers))
    if summa % 11 % 10 != checksum:
        raise ValidationError(u'Не пройдена проверка контрольного числа')


def inn10_validator(value):
    u"""Валидатор для ИНН юридического лица (10 цифр).

    Проверяет корректность формата и контрольное число (последняя цифра).

    :raises django.core.exceptions.ValidationError: Если аргумент value
        содержит значение, несоответствующее формату ИНН, либо не пройдена
        проверка контрольного числа.
    """
    value = unicode(value)

    if len(value) != 10 and not value.isdigit():
        raise ValidationError(u'ИНН должен быть 10-тизначным числом')

    _check_inn_checksum((2, 4, 10, 3, 5, 9, 4, 6, 8),
                        (int(ch) for ch in value[:-1]),
                        int(value[-1]))


def inn12_validator(value):
    u"""Валидатор для ИНН физического лица (12 цифр).

    Проверяет корректность формата и контрольное число (последние 2 цифры).

    :raises django.core.exceptions.ValidationError: Если аргумент value
        содержит значение, несоответствующее формату ИНН, либо не пройдена
        проверка контрольного числа.
    """
    value = unicode(value)

    if len(value) != 12 and not value.isdigit():
        raise ValidationError(u'ИНН должен быть 12-тизначным числом')

    _check_inn_checksum((7, 2, 4, 10, 3, 5, 9, 4, 6, 8),
                        (int(ch) for ch in value[:-2]),
                        int(value[-2]))

    _check_inn_checksum((3, 7, 2, 4, 10, 3, 5, 9, 4, 6, 8),
                        (int(ch) for ch in value[:-1]),
                        int(value[-1]))


def inn_validator(value):
    u"""Валидатор для поля ИНН.

    Проверяет корректность формата и контрольное число (1 или 2 последние
    цифры).

    :raises django.core.exceptions.ValidationError: Если аргумент value
        содержит значение, несоответствующее формату ИНН, либо не пройдена
        проверка контрольного числа.
    """
    value = unicode(value)

    if len(value) not in (10, 12) or not value.isdigit():
        raise ValidationError(u'ИНН должен быть 10-ти или 12-тизначным числом')

    if len(value) == 10:
        inn10_validator(value)
    else:
        inn12_validator(value)


is_inn_valid = partial(validate_value, validator=inn_validator)
is_inn10_valid = partial(validate_value, validator=inn10_validator)
is_inn12_valid = partial(validate_value, validator=inn12_validator)
